from .appearance   import AppearancePageContent
from .fonts        import FontsPageContent
from .top_bar      import TopBarPageContent
from .sound        import SoundPageContent
from .pointing     import PointingPageContent
from .display      import DisplayPageContent
from .login_screen import LoginScreenPageContent
from .tools        import ToolsPageContent
from .power        import PowerPageContent
